const { workspace } = require('../helpers');
const { promisify } = require('util');

const generateModules = (injectables) => {
  const { moduleKeys, envRun, creationConfig } = injectables;

  if (moduleKeys.length === 0) {
    return Promise.resolve();
  }

  const [nextModuleKey, ...restModules] = moduleKeys;
  const props = creationConfig.module[nextModuleKey];

  return envRun(`module/${nextModuleKey}`, { props }).then(() => {
    if (restModules.length > 0) {
      return generateModules({
        ...injectables,
        moduleKeys: restModules,
      });
    } else {
      return Promise.resolve();
    }
  });
};

const generateDependants = (injectables) => () => {
  const { env, creationConfig } = injectables;
  const envRun = promisify(env.run).bind(env);
  const moduleKeys = Object.keys(creationConfig.module);
  const newInjectable = {
    ...injectables,
    envRun,
    moduleKeys,
  };

  return Promise.all([generateModules(newInjectable)]);
};

const generateDependencyApp = (injectables) => {
  const { env, creationConfig } = injectables;
  const envRun = promisify(env.run).bind(env);
  const applicationKey = Object.keys(creationConfig.application)[0];
  const props = creationConfig.application[applicationKey];
  return envRun(`application/${applicationKey}`, { props });
};

module.exports = (injectables) => {
  const creationConfig = workspace.getCreationConfig(
    injectables.program.config,
  );
  const injectableWithCreationConfig = { ...injectables, creationConfig };

  generateDependencyApp(injectableWithCreationConfig).then(
    generateDependants(injectableWithCreationConfig),
  );
};
