'use strict';
const Generator = require('yeoman-generator');
const prompts = require('./prompts');
const folderScanner = require('folder-scanner');
const helpers = require('../../../helpers');
const changeCase = require('change-case');
const path = require('path');

module.exports = class extends Generator {
	constructor(...args) {
		super(...args);
		this._configureOptions();
	}

	prompting() {
		if (!helpers.object.isEmpty(this.options.props)) {
			this.props = this.options.props;
			return;
		}

		return this.prompt(prompts).then((props) => {
			this.props = props;
		});
	}

	configuring() {
		this.props = this.props || {};
		this.props.titleCaseAppName = changeCase.titleCase(this.props.appName);
		this.props.paramCaseAppName = changeCase.paramCase(this.props.appName);
		helpers.color.parseColorSelection(this.props);
	}

	writing() {
		const templatePath = this.templatePath();
		const filePaths = folderScanner.scanPath(templatePath);
		this._copyFiles(filePaths, templatePath);
	}

	install() {
		switch (this.props.packageManager) {
			case 'npm':
				this.npmInstall();
				break;
			case 'yarn':
				this.yarnInstall();
				break;
			default:
				this.log('No package manager selected.');
		}
	}

	/**
   * Method executed on the constructor to
   * configure options received by the cli.
   */
	_configureOptions() {
		this.option('props', { type: Object, default: null });
	}

	/**
   * @param {String[]} filePaths
   * @param {String} templatePath
   */
	_copyFiles(filePaths, templatePath) {
		filePaths.forEach((filePath) => {
			this._copyFileTo(templatePath, filePath);
		});
	}

	/**
   *
   * @param {String} templatePath
   * @param {String} filePath
   */
	_copyFileTo(templatePath, filePath) {
		const relativeFilePath = path.relative(templatePath, filePath);
		this.fs.copyTpl(
			filePath,
			this.destinationPath(relativeFilePath.replace('.ejs', '')),
			this.props
		);
	}
};
