'use strict';
const Generator = require('yeoman-generator');
const prompts = require('./prompts');
const { parameters, moduleConstants } = require('../../../constants');

module.exports = class extends Generator {
  prompting() {
    return this.prompt(prompts).then((props) => {
      this.props = props;
    });
  }

  writing() {
    const defaultName = parameters.CONFIG_FILE_DEFAULT_NAME;
    this._createConfigurationFile(defaultName);
  }

  _buildJson() {
    let response = {
      application: this._getApplicationConfiguration(),
    };

    if (this.props.modulesToInstall.length > 0) {
      response = {
        ...response,
        module: this._getModulesConfiguration(this.props),
      };
    }

    return response;
  }

  _getApplicationConfiguration() {
    switch (this.props.applicationType) {
      case 'site':
        return {
          site: {
            appName: 'app',
            primaryColor: 'white',
            accentColor: 'blue',
            packageManager: 'npm',
            baseApiUrl: '127.0.0.1',
          },
        };

      case 'enrollment':
        return {
          enrollment: {
            appName: 'app',
            primaryColor: 'white',
            accentColor: 'pink',
            packageManager: 'npm',
            baseApiUrl: '127.0.0.1',
            limitLandingWidth: false,
            stepsInFooter: 3,
          },
        };
      default:
        return {};
    }
  }

  _getModulesConfiguration(props) {
    return props.modulesToInstall.reduce((acum, item, index) => {
      return {
        ...acum,
        [item]: moduleConstants.defaultPrompts[item] || {},
      };
    }, {});
  }

  _createConfigurationFile(defaultName) {
    const jsonFile = this._buildJson();
    this.fs.writeJSON(this.destinationPath(defaultName), jsonFile);
  }
};
