const path = require('path');
const fs = require('fs');
const fse = require('fs-extra');
const errors = require('../classes/errors');
const WORKSPACE_FILENAME = '.c6-workspace.json';
const constants = require('../constants');

module.exports = {
  getCreationConfig(fileName) {
    return fse.readJSONSync(
      path.join(
        process.cwd(),
        fileName || constants.parameters.CONFIG_FILE_DEFAULT_NAME,
      ),
    );
  },
  verifyWorkspaceFolder(folderPath) {
    if (this.workspaceFileNotExist(folderPath)) {
      throw new errors.NotAWorkspaceError();
    }
  },
  verifyConfigFolder(folderPath) {
    const workspaceConfig = this.getWorkspaceConfig(folderPath);
    if (this.configurationFolderNotExist(folderPath, workspaceConfig)) {
      throw new errors.FolderNotExistError(
        workspaceConfig.configFilesPath,
        "please verify 'configFilesPath' is correct in workspace.",
      );
    }
  },
  workspaceFileNotExist(folderPath) {
    return !fs.existsSync(path.resolve(folderPath, WORKSPACE_FILENAME));
  },
  configurationFolderNotExist(folderPath, workspaceConfig) {
    workspaceConfig = workspaceConfig || this.getWorkspaceConfig(folderPath);
    return !fs.existsSync(
      path.resolve(folderPath, workspaceConfig.configFilesPath),
    );
  },
  getWorkspaceConfig(folderPath) {
    this.verifyWorkspaceFolder(folderPath);
    return JSON.parse(
      fs.readFileSync(path.resolve(folderPath, WORKSPACE_FILENAME), 'utf8'),
    );
  },
  configurationFileExist(fileName) {
    const currentDirectory = process.cwd();
    return fs.existsSync(
      path.join(currentDirectory, fileName || '.c6-config.json'),
    );
  },
};
