#!/usr/bin/env node
'use strict';
const program = require('commander');
const yeoman = require('yeoman-environment');
const env = yeoman.createEnv();
const { c6ConfigAction } = require('./actions');
const { configurationFileExist } = require('./helpers/workspace.helper');
const { registrateGenerators } = require('./helpers/generator.helper');
const pkg = require('./package.json');

let injectables = {
  env,
};

registrateGenerators(injectables);

program
  .version(pkg.version)
  .option('-c --config <config>', 'configuration file name.')
  .option('-cc --create-config', 'create a configuration file name.')
  .option(
    '-s --subgenerator <subgenerator>',
    'execute a subgenerator directly.',
  );

program.parse(process.argv);

injectables = {
  ...injectables,
  program,
};

if (program.args.length === 0) {
  if (program.config || configurationFileExist()) {
    c6ConfigAction(injectables);
    return;
  }

  env.run('c6', {
    config: program.config,
    'create-config': program.createConfig,
    subgenerator: program.subgenerator,
  });
}
